This is the mail archive of the gdb-patches@sourceware.org mailing list for the GDB project.


Index Nav: [Date Index] [Subject Index] [Author Index] [Thread Index]
Message Nav: [Date Prev] [Date Next] [Thread Prev] [Thread Next]
Other format: [Raw text]

[RFC 2/7] Add libiberty/concat styled concat_path function


    This commit adds concat_path function to concatenate an arbitrary number of
    path elements.  The function automatically adds an directory separator between
    two elements as needed.

    gdb/ChangeLog:

	* common/common-utils (startswith): Convert to C++.
	(endswith): New function.
	* utils.c (_concat_path, approx_path_length): New function.
	* utils.h (_concat_path): New export.
	(concat_path): New define.
---
 gdb/common/common-utils.h | 16 +++++++++++++---
 gdb/utils.c               | 46 ++++++++++++++++++++++++++++++++++++++++++++++
 gdb/utils.h               | 17 +++++++++++++++++
 3 files changed, 76 insertions(+), 3 deletions(-)

diff --git a/gdb/utils.c b/gdb/utils.c
index ee37e49..e924b2c 100644
--- a/gdb/utils.c
+++ b/gdb/utils.c
@@ -3193,6 +3193,52 @@ substitute_path_component (std::string &str, const std::string &from,
     }
 }
 
+/* Approximate length of final path.  Helper for concat_path.  */
+
+static inline unsigned long
+approx_path_length (std::initializer_list<std::string> args,
+		   std::string dir_separator)
+{
+  size_t length = 0;
+
+  for (std::string arg: args)
+      length += arg.length () + dir_separator.length ();
+
+  return length;
+}
+
+/* See utils.h.  */
+
+std::string
+_concat_path (std::initializer_list<std::string> args,
+	      std::string dir_separator)
+{
+  std::string dst;
+  dst.reserve (approx_path_length (args, dir_separator));
+
+  for (std::string arg: args)
+    {
+      if (arg.empty ())
+	continue;
+
+      if (startswith (arg, dir_separator)
+	  && endswith (dst, dir_separator))
+	dst.erase (dst.length () - dir_separator.length (),
+		   dir_separator.length ());
+
+      else if (!dst.empty ()
+	       && !startswith (arg, dir_separator)
+	       && !endswith (dst, dir_separator)
+	       && dst != TARGET_SYSROOT_PREFIX)
+	dst += dir_separator;
+
+      dst += arg;
+    }
+
+  dst.shrink_to_fit ();
+  return dst;
+}
+
 #ifdef HAVE_WAITPID
 
 #ifdef SIGALRM
diff --git a/gdb/utils.h b/gdb/utils.h
index 674f672..5d68a18 100644
--- a/gdb/utils.h
+++ b/gdb/utils.h
@@ -24,6 +24,7 @@
 #include "exceptions.h"
 #include "common/scoped_restore.h"
 #include <chrono>
+#include <string>
 
 extern void initialize_utils (void);
 
@@ -143,6 +144,22 @@ extern void substitute_path_component (std::string &str,
 				       const std::string &from,
 				       const std::string &to);
 
+/* Concatenate an arbitrary number of path elements.  Adds and removes
+   directory separators as needed.
+
+   concat_path (/first, second)		=> /first/second
+   concat_path (first, second)		=> first/second
+   concat_path (first/, second)		=> first/second
+   concat_path (first, /second)		=> first/second
+   concat_path (first/, /second)	=> first/second
+   concat_path (target:, second)	=> target:second
+   */
+
+extern std::string _concat_path (std::initializer_list<std::string> args,
+				 std::string dir_separator);
+
+#define concat_path(...) _concat_path ({__VA_ARGS__}, SLASH_STRING)
+
 char *ldirname (const char *filename);
 
 extern int count_path_elements (const char *path);
diff --git a/gdb/common/common-utils.h b/gdb/common/common-utils.h
index 618d266..be3c106 100644
--- a/gdb/common/common-utils.h
+++ b/gdb/common/common-utils.h
@@ -74,13 +74,23 @@ char *savestring (const char *ptr, size_t len);
 
 extern char *safe_strerror (int);
 
 /* Return non-zero if the start of STRING matches PATTERN, zero
    otherwise.  */
 
 static inline int
-startswith (const char *string, const char *pattern)
+startswith (std::string str, std::string pattern)
 {
-  return strncmp (string, pattern, strlen (pattern)) == 0;
+  return (str.find (pattern) == 0);
+}
+
+/* Return non-zero if the end of STR matches PATTERN, zero
+   otherwise.  */
+
+static inline int
+endswith (std::string str, std::string pattern)
+{
+  return (str.rfind (pattern) != std::string::npos
+	  && str.rfind (pattern) == (str.length () - pattern.length ()));
 }
 
 ULONGEST strtoulst (const char *num, const char **trailer, int base);
@@ -74,13 +74,23 @@ char *savestring (const char *ptr, size_t len);
 extern char *safe_strerror (int);
 
-/* Return non-zero if the start of STRING matches PATTERN, zero
+/* Return non-zero if the start of STR matches PATTERN, zero
    otherwise.  */
 
-- 
2.8.4


Index Nav: [Date Index] [Subject Index] [Author Index] [Thread Index]
Message Nav: [Date Prev] [Date Next] [Thread Prev] [Thread Next]